-- Author: U_BMP
-- Group: vk.com/https://vk.com/biomodprod_utilit_fs
-- Date: 07.05.2025

VehicleBeehive = {}
VehicleBeehive.modName = g_currentModName or "unknown"

-- Проверка зависимостей
function VehicleBeehive.prerequisitesPresent(specializations)
    return true
end

-- Инициализация специализации
function VehicleBeehive.initSpecialization()
    local schema = Vehicle.xmlSchema
    schema:setXMLSpecializationType("VehicleBeehive")
    VehicleBeehive.registerXMLPaths(schema, "vehicle.vehicleBeehive")
    schema:setXMLSpecializationType()

    -- Регистрация пути для сохранения с учётом modName
    local schemaSavegame = Vehicle.xmlSchemaSavegame
    local savegamePath = string.format("vehicles.vehicle(?).%s.vehicleBeehive#pendingHoney", VehicleBeehive.modName)
    schemaSavegame:register(XMLValueType.FLOAT, savegamePath, "Current amount of honey in the beehive (liters)", 0)
    print("[VehicleBeehive] Registered savegame path: " .. savegamePath)
end

-- Регистрация XML-путей
function VehicleBeehive.registerXMLPaths(schema, basePath)
    print("[VehicleBeehive] Registering XML paths for basePath: " .. basePath)
    schema:register(XMLValueType.FLOAT, basePath .. "#actionRadius", "Bees action radius", 25)
    schema:register(XMLValueType.FLOAT, basePath .. "#litersHoneyPerDay", "Beehive honey production per active day", 10)
    schema:register(XMLValueType.FLOAT, basePath .. "#transferRadius", "Radius for transferring honey to spawner (meters)", 10)
    schema:register(XMLValueType.FLOAT, basePath .. "#capacity", "Maximum honey capacity of the beehive (liters)", 1000)
    schema:register(XMLValueType.FLOAT, basePath .. "#minTransferAmount", "Minimum honey amount to transfer to spawner (liters)", 10)
    schema:register(XMLValueType.FLOAT, basePath .. "#pendingHoney", "Current amount of honey in the beehive (liters)", 0)
    
    -- Эффекты и звуки
    EffectManager.registerEffectXMLPaths(schema, basePath .. ".effects")
    SoundManager.registerSampleXMLPaths(schema, basePath .. ".sounds", "idle")
    
    -- Табло мёда
    schema:register(XMLValueType.NODE_INDEX, basePath .. ".displayHoney(?)#node", "Display honey amount node")
    schema:register(XMLValueType.STRING, basePath .. ".displayHoney(?)#font", "Display font name", "DIGIT")
    schema:register(XMLValueType.STRING, basePath .. ".displayHoney(?)#alignment", "Display text alignment", "CENTER")
    schema:register(XMLValueType.FLOAT, basePath .. ".displayHoney(?)#size", "Display text size", 0.13)
    schema:register(XMLValueType.FLOAT, basePath .. ".displayHoney(?)#scaleX", "Display text x scale", 1)
    schema:register(XMLValueType.FLOAT, basePath .. ".displayHoney(?)#scaleY", "Display text y scale", 1)
    schema:register(XMLValueType.STRING, basePath .. ".displayHoney(?)#mask", "Display text mask", "000.0")
    schema:register(XMLValueType.FLOAT, basePath .. ".displayHoney(?)#emissiveScale", "Display emissive scale", 1)
    schema:register(XMLValueType.COLOR, basePath .. ".displayHoney(?)#color", "Display text color", "1 0.5 0")
    schema:register(XMLValueType.COLOR, basePath .. ".displayHoney(?)#hiddenColor", "Display text hidden color", "0.05 0.05 0.05")
end

-- Регистрация функций
function VehicleBeehive.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getBeehiveInfluenceFactor", VehicleBeehive.getBeehiveInfluenceFactor)
    SpecializationUtil.registerFunction(vehicleType, "updateBeehiveState", VehicleBeehive.updateBeehiveState)
    SpecializationUtil.registerFunction(vehicleType, "getHoneyAmountToSpawn", VehicleBeehive.getHoneyAmountToSpawn)
    SpecializationUtil.registerFunction(vehicleType, "transferHoneyToSpawner", VehicleBeehive.transferHoneyToSpawner)
    SpecializationUtil.registerFunction(vehicleType, "setHoneyDisplay", VehicleBeehive.setHoneyDisplay)
end

-- Регистрация событий
function VehicleBeehive.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", VehicleBeehive)
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", VehicleBeehive)
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", VehicleBeehive)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", VehicleBeehive)
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", VehicleBeehive)
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", VehicleBeehive)
end

-- Загрузка объекта
function VehicleBeehive:onLoad(savegame)
    self.spec_beehive = {}
    local spec = self.spec_beehive
    local xmlFile = self.xmlFile

    spec.environment = g_currentMission.environment
    spec.isFxActive = false
    spec.isProductionActive = false
    spec.actionRadius = xmlFile:getValue("vehicle.vehicleBeehive#actionRadius", 25)
    
    -- Проверка на nil для litersHoneyPerDay
    local litersHoneyPerDay = xmlFile:getValue("vehicle.vehicleBeehive#litersHoneyPerDay", 10)
    spec.honeyPerHour = litersHoneyPerDay and litersHoneyPerDay / 24 or 10 / 24
    spec.actionRadiusSquared = spec.actionRadius ^ 2
    spec.pendingHoney = xmlFile:getValue("vehicle.vehicleBeehive#pendingHoney", 0)
    spec.dirtyFlag = self:getNextDirtyFlag()
    
    -- Загрузка pendingHoney из сохранения
    if savegame ~= nil then
        local savegameKey = string.format("%s.%s.vehicleBeehive", savegame.key, VehicleBeehive.modName)
        spec.pendingHoney = savegame.xmlFile:getValue(savegameKey .. "#pendingHoney", spec.pendingHoney)
        print("[VehicleBeehive] Loaded pendingHoney from savegame: " .. spec.pendingHoney)
    end
    
    -- Новые параметры
    spec.transferRadius = xmlFile:getValue("vehicle.vehicleBeehive#transferRadius", 10)
    spec.capacity = xmlFile:getValue("vehicle.vehicleBeehive#capacity", 1000)
    spec.minTransferAmount = xmlFile:getValue("vehicle.vehicleBeehive#minTransferAmount", 10)

    spec.lastDayTimeHoneySpawned = spec.environment.dayTime
    spec.lastPosition = {getWorldTranslation(self.rootNode)}
    
    -- Загрузка эффектов и звуков (только для клиента)
    if self.isClient then
        spec.effects = g_effectManager:loadEffect(xmlFile, "vehicle.vehicleBeehive.effects", self.components, self, self.i3dMappings)
        g_effectManager:setEffectTypeInfo(spec.effects, FillType.UNKNOWN)
        spec.samples = {}
        spec.samples.idle = g_soundManager:loadSampleFromXML(xmlFile, "vehicle.vehicleBeehive.sounds", "idle", self.baseDirectory, self.components, 1, AudioGroup.ENVIRONMENT, self.i3dMappings, nil)
    end
    
    -- Загрузка дисплеев мёда
    spec.honeyDisplays = {}
    xmlFile:iterate("vehicle.vehicleBeehive.displayHoney", function(_, key)
        local displayNode = xmlFile:getValue(key .. "#node", nil, self.components, self.i3dMappings)
        if displayNode then
            local fontName = xmlFile:getValue(key .. "#font", "DIGIT"):upper()
            local fontMaterial = g_materialManager:getFontMaterial(fontName, self.customEnvironment)
            if fontMaterial then
                local display = {}
                local alignment = xmlFile:getValue(key .. "#alignment", "CENTER")
                local textAlignment = RenderText["ALIGN_" .. alignment:upper()] or RenderText.ALIGN_CENTER
                local size = xmlFile:getValue(key .. "#size", 0.13)
                local scaleX = xmlFile:getValue(key .. "#scaleX", 1)
                local scaleY = xmlFile:getValue(key .. "#scaleY", 1)
                local mask = xmlFile:getValue(key .. "#mask", "000.0")
                local emissiveScale = xmlFile:getValue(key .. "#emissiveScale", 1)
                local color = xmlFile:getValue(key .. "#color", {1, 0.5, 0}, true)
                local hiddenColor = xmlFile:getValue(key .. "#hiddenColor", {0.05, 0.05, 0.05}, true)
                
                display.displayNode = displayNode
                display.characterLine = CharacterLine.new(displayNode, fontMaterial, mask:len())
                display.characterLine:setSizeAndScale(size, scaleX, scaleY)
                display.characterLine:setTextAlignment(textAlignment)
                display.characterLine:setColor(color, hiddenColor, emissiveScale)
                
                table.insert(spec.honeyDisplays, display)
            else
                print("[VehicleBeehive] Failed to load font material for displayHoney: " .. fontName)
            end
        end
    end)
    
    -- Устанавливаем начальное значение дисплея
    if self.isClient then
        self:setHoneyDisplay(spec.pendingHoney)
    end
end

-- Сохранение в XML
function VehicleBeehive:saveToXMLFile(xmlFile, key, usedModNames)
    local spec = self.spec_beehive
    local savegameKey = key .. "#pendingHoney"
    print("[VehicleBeehive] Saving to " .. savegameKey .. ", value: " .. spec.pendingHoney)
    xmlFile:setValue(savegameKey, spec.pendingHoney)
end

-- После загрузки
function VehicleBeehive:onPostLoad(savegame)
    g_currentMission.beehiveSystem:addBeehive(self)
    self:updateBeehiveState()
end

-- Удаление объекта
function VehicleBeehive:onDelete()
    local spec = self.spec_beehive
    if self.isClient then
        g_effectManager:deleteEffects(spec.effects)
        g_soundManager:deleteSamples(spec.samples)
        for _, display in ipairs(spec.honeyDisplays or {}) do
            if display.characterLine and type(display.characterLine.delete) == "function" then
                display.characterLine:delete()
            end
        end
    end
    g_currentMission.beehiveSystem:removeBeehive(self)
end

-- Обновление координат при движении
function VehicleBeehive:onUpdate(dt)
    local spec = self.spec_beehive
    local raiseActive = false

    if self.isServer then
        local wx, _, wz = getWorldTranslation(self.rootNode)
        local dx = wx - spec.lastPosition[1]
        local dz = wz - spec.lastPosition[3]
        if dx * dx + dz * dz > 0.1 then -- Проверяем, переместился ли трейлер
            spec.wx = wx
            spec.wz = wz
            spec.lastPosition = {wx, spec.lastPosition[2], wz}
            raiseActive = true
        end

        -- Производство мёда
        if spec.isProductionActive then
            local honeyAmount = self:getHoneyAmountToSpawn()
            if honeyAmount > 0 then
                spec.pendingHoney = math.min(spec.pendingHoney + honeyAmount, spec.capacity)
                self:transferHoneyToSpawner()
                self:raiseDirtyFlags(spec.dirtyFlag)
                raiseActive = true
            end
        end
    end

    -- Обновление дисплея
    if self.isClient then
        self:setHoneyDisplay(spec.pendingHoney)
    end

    if raiseActive then
        self:raiseActive()
    end
end

-- Синхронизация через поток
function VehicleBeehive:onReadStream(streamId, connection)
    local spec = self.spec_beehive
    spec.pendingHoney = streamReadFloat32(streamId)
    spec.isProductionActive = streamReadBool(streamId)
    spec.isFxActive = streamReadBool(streamId)
end

function VehicleBeehive:onWriteStream(streamId, connection)
    local spec = self.spec_beehive
    streamWriteFloat32(streamId, spec.pendingHoney)
    streamWriteBool(streamId, spec.isProductionActive)
    streamWriteBool(streamId, spec.isFxActive)
end

-- Фактор влияния пчёл
function VehicleBeehive:getBeehiveInfluenceFactor(x, z)
    local spec = self.spec_beehive
    local distanceSquared = MathUtil.getPointPointDistanceSquared(spec.wx, spec.wz, x, z)
    return distanceSquared > spec.actionRadiusSquared and 0 or 1 - distanceSquared * 0.85 / spec.actionRadiusSquared
end

-- Обновление состояния улья
function VehicleBeehive:updateBeehiveState()
    local spec = self.spec_beehive
    local beehiveSystem = g_currentMission.beehiveSystem
    spec.isProductionActive = beehiveSystem.isProductionActive
    if spec.isFxActive ~= beehiveSystem.isFxActive then
        spec.isFxActive = beehiveSystem.isFxActive
        if self.isClient then
            if spec.isFxActive then
                g_effectManager:startEffects(spec.effects)
                g_soundManager:playSample(spec.samples.idle, 0)
            else
                g_effectManager:stopEffects(spec.effects)
                g_soundManager:stopSample(spec.samples.idle)
            end
        end
    end
end

-- Расчёт количества мёда
function VehicleBeehive:getHoneyAmountToSpawn()
    local spec = self.spec_beehive
    if not spec.isProductionActive then
        return 0
    end
    local hoursPassed = (spec.environment.dayTime - spec.lastDayTimeHoneySpawned) / 1000 / 60 / 60
    local timeFactor = math.min(math.abs(hoursPassed), 1)
    local honeyAmount = spec.honeyPerHour * timeFactor * g_currentMission.environment.timeAdjustment
    spec.lastDayTimeHoneySpawned = spec.environment.dayTime
    return honeyAmount
end

-- Передача мёда в спавнер
function VehicleBeehive:transferHoneyToSpawner()
    local spec = self.spec_beehive
    if not self.isServer or spec.pendingHoney < spec.minTransferAmount then
        return
    end

    local farmId = self:getOwnerFarmId()
    local spawner = g_currentMission.beehiveSystem:getFarmBeehivePalletSpawner(farmId)
    if spawner then
        -- Проверка расстояния до спавнера
        local spawnerX, _, spawnerZ = getWorldTranslation(spawner.rootNode)
        local distanceSquared = MathUtil.getPointPointDistanceSquared(spec.wx, spec.wz, spawnerX, spawnerZ)
        if distanceSquared <= spec.transferRadius * spec.transferRadius then
            spawner:addFillLevel(spec.pendingHoney)
            spec.pendingHoney = 0
            self:raiseDirtyFlags(spec.dirtyFlag)
        end
    end
end

-- Обновление дисплея мёда
function VehicleBeehive:setHoneyDisplay(amount)
    local spec = self.spec_beehive
    amount = math.floor(amount * 10) / 10 -- Округление до 1 знака после запятой
    for _, display in ipairs(spec.honeyDisplays or {}) do
        display.characterLine:setText(string.format("%.1f", amount))
    end
end